/*
 * Copyright by Marco Savegnago.  All rights reserved.
 * This software may be freely distributed and used, but it may not
 * under any circumstances be sold by anyone other than the author.
 * It may be distributed by a commercial company as long as it is
 * for no cost.
 *
 ******************************* Creazione file *******************************
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago             	Data di creazione : 21/1/2000
 * Codice   :
 *
 * AX25 Address class definition
 */

#ifndef  AX25_H
#define  AX25_H

#include    "tdef.h"

const	CMAXCLEN		=	6;		// # of characters in callsign
const	CMAXALEN		=	7;		// # of characters in full ax25 address
const	CMAXDIGIS    	=	8;  	// max digipeaters in path
const	CMAXFRAMELEN 	=	256;	// max frame len
const	CMINFRAMELEN 	=	15;		// min frame len
const	CAX25ECLEN		=	6;		// # of char in encoded AX25 callsign
const	CAX25DCLEN		=	10;		// # of char in decoded AX25 callsign

const CAX25AddressFlag_E_Mask					= 0x01;
const CAX25AddressFlag_SSID_Mask				= 0x1e;
const CAX25AddressFlag_reserved_Mask			= 0x60;
const CAX25AddressFlag_H_Mask					= 0x80;
const CAX25AddressFlag_C_Mask					= 0x80;

const CAX25AddressShift                			= 1;
const CAX25AddressSSIDMax              			= 15;
const CAX25AddressWildSSID						= 0xff;


struct TAX25Address
{
	byte    callsign[CMAXCLEN];      /* Callsign, shifted left one bit 	*/
	byte    ssid;                   /* sub-station ID field 		*/
	/*******************************************************************
		Default constructor for an AX25 address type.
	*******************************************************************/

	TAX25Address();
	TAX25Address(const char* address);
	TAX25Address(const byte* address);

	/*******************************************************************
		Copy constructor for an AX25 address type.
	*******************************************************************/

	TAX25Address(const TAX25Address& other);

	TAX25Address& operator =(const TAX25Address& other);
	TAX25Address& operator =(const char *other);

	char*		getAX25Address(char *address, int withSSID=true) const;
	byte*		getAX25Address(byte *address, int maxLen=CMAXALEN);

	int 		setAX25Address(const char *address);
	void 		setAX25Address(const byte *address);

	int			compare(const TAX25Address& compareWith, int withSSID=true) const;
	int			compare(const char* compareWith, int withSSID=true);
	int 		operator == ( const TAX25Address& compareWith ) const;
	int 		operator != ( const TAX25Address& compareWith ) const;
	int			operator<	( const TAX25Address& compareWith ) const;

	static 	int	compare(const TAX25Address& one, void* item);
	static 	int	compareOnlyCall(const TAX25Address& one, void* item);

	static  	int	compare(void* first, void* second);
	static  	int validCallsign(const byte* call);

				int	IsValid();
				int	IsNull();

				byte*	getRawAddress(byte* buffer);
				word	getLen();

	static	const char AX25AddressWildCards[3];
	static	const char AX25AddressWildCardsShifted[3];
};

inline TAX25Address::TAX25Address()
{
	*callsign=ssid=EOS;
}

inline int TAX25Address::operator == (const TAX25Address& compareWith ) const
{
	return compare(compareWith, true);
}

inline int TAX25Address::operator != (const TAX25Address& compareWith ) const
{
	return int(compare(compareWith, true)==false);
}

inline int	TAX25Address::IsValid()
{
	return validCallsign(callsign);
}

inline int	TAX25Address::IsNull()
{
	return int(*callsign == EOS);
}

inline	word TAX25Address::getLen()
{
	return (sizeof(callsign) + sizeof(ssid));
}

#endif

